import json
import zipfile
from pathlib import Path
from typing import Type, Dict, Any, Generator, Union, List

from progress.bar import Bar
from pydantic import BaseModel
from pydantic.dataclasses import dataclass
from pyimporters_plugins.base import (
    KnowledgeParserOptions,
    KnowledgeParserBase,
    Term,
    maybe_archive,
)


@dataclass
class JSONOptions(KnowledgeParserOptions):
    """
    Options for the JSON knowledge import
    """

    pass


JSONOptionsModel = JSONOptions.__pydantic_model__


class JSONKnowledgeParser(KnowledgeParserBase):
    def parse(
        self, source: Path, options: Union[BaseModel, Dict[str, Any]], bar: Bar
    ) -> Generator[Term, None, None]:
        options = JSONOptionsModel(**options) if isinstance(options, dict) else options
        with maybe_archive(source, mode="rb") as fin:
            if isinstance(fin, zipfile.ZipFile):
                for zentry in fin.filelist:
                    with fin.open(zentry) as zfin:
                        yield from self._parse_json(bar, zfin)
            else:
                yield from self._parse_json(bar, fin)

    def _parse_json(self, bar, fin):
        terms = json.load(fin)
        bar.max = len(terms)
        bar.start()
        for term in terms:
            bar.next()
            yield Term(**term)

    @classmethod
    def get_schema(cls) -> KnowledgeParserOptions:
        return JSONOptions

    @classmethod
    def get_model(cls) -> Type[BaseModel]:
        return JSONOptionsModel

    @classmethod
    def get_extensions(cls) -> List[str]:
        return ["json", "zip"]



from pathlib import Path

from progress.bar import Bar
from pyimporters_plugins.base import Term

from pyimporters_json.json_sherpa import JSONKnowledgeParser, JSONOptionsModel


def test_json():
    testdir = Path(__file__).parent
    source = Path(testdir, "data/organization_en.json")
    parser = JSONKnowledgeParser()
    options = JSONOptionsModel()
    concepts = list(parser.parse(source, options.dict(), Bar("Processing")))
    assert len(concepts) == 1527
    c7: Term = concepts[7]
    assert c7.identifier == "afporganization:2021"
    assert c7.preferredForm == "REYNOLDS AMERICAN"
    assert len(c7.properties["altForms"]) == 1
    assert "Reynolds American" in c7.properties["altForms"]


def test_zip():
    testdir = Path(__file__).parent
    source = Path(testdir, "data/tmpymh9kthe.zip")
    parser = JSONKnowledgeParser()
    options = JSONOptionsModel()
    concepts = list(parser.parse(source, options.dict(), Bar("Processing")))
    assert len(concepts) == 3
    c0: Term = concepts[0]
    assert c0.identifier == "afpperson:1698"
    assert c0.properties["status"] == "W"
    c2: Term = concepts[2]
    assert c2.identifier == "afpperson:999999"
    assert c2.properties["status"] == "M"
